# ANSI termsequences handling

use strict;
use utf8;
no strict 'refs';

our $verbose;

my $page_fg;		# terminal state -- current color and font style
my $page_bg;
my $page_font;

# print_t($string)
# print_c($string)

sub print_line_ansi
{	
	my $func;

	foreach my $chunk (split(/([\n\r\t\x0F\x08\x0C]|\033\[\??[0-9;]*[a-zA-Z]|\033[78=>Fclmno|}~])/, shift)) {
		if($chunk =~ /^\033\[(.*)([a-zA-Z])/) {
			$func = "print_csi_$2";
			&$func(split(';', $1)) if defined &$func;
			warn "unknown ANSI seq CSI $1$2\n" if $verbose && !defined &$func;
		} elsif($chunk =~ /^\033\[\?(.*)([a-zA-Z])/) {
			$func = "print_csi_q_$2";
			&$func(split(';', $1)) if defined &$func;
			warn "unknown ANSI seq CSI ? $1$2\n" if $verbose && !defined &$func;
		} elsif($chunk =~ /^\033(.)/) {
			# ignored
		} elsif($chunk eq "\x0C") {
			# form feed
			new_line(1);
		} elsif($chunk eq "\x0F") {
			# ignored
		} elsif($chunk eq "\n") {
			# ignored
		} elsif($chunk eq "\r") {
			put_cr();
		} elsif($chunk eq "\t") {
			put_tab();
		} elsif($chunk eq "\x08") {
			put_bs();
		} elsif($chunk eq "") {
			# do nothing
		} else {
			put_text($chunk);
		}
	}
	return 0;
}

# this is called on the start of page to init
# postscript state according to terminal state
sub page_setattrs
{
	print ifdef($page_font, "m0 ");
	print $page_bg if defined $page_bg;
	print $page_fg if defined $page_fg;
}

sub print_csi_m
{
	push(@_, 0) unless @_;
	foreach my $c (@_) {
		# force numeric context
		$c += 0;
		if($c == 0 || $c == 1 || $c == 4 || $c == 5 
			|| $c == 22 || $c == 24 || $c == 25
			|| ($c >= 30 && $c <= 39)
			|| ($c >= 40 && $c <= 49))
		{
			print "m$c ";
		}
		if($c == 0) {
			undef $page_fg;
			undef $page_bg;
			undef $page_font;
		} elsif($c == 1 || $c == 4 || $c == 22 || $c == 24 || $c == 25) {
			$page_font = "m$c ";
		} elsif($c >= 30 && $c <= 39) {
			$page_fg = "m$c ";
		} elsif($c >= 40 && $c <= 49) {
			$page_bg = "m$c ";
		};
	}
}

sub print_csi_H
{
	return unless @_ == 2;
	printf "\n%s %s H ", @_;
	#$t_line = $_[0];
}

sub print_csi_A { printf "%s A ", shift }
sub print_csi_B { printf "%s B ", shift }
sub print_csi_C { printf "%s C ", shift }
sub print_csi_D { printf "%s D ", shift }

1;
