# Files handling & running u2ps_{rs,pp}

use POSIX qw(WIFEXITED WEXITSTATUS WIFSIGNALED WTERMSIG WIFSTOPPED);
use utf8;

our $o_gs;
our $b_px;
our $o_keep;
our $o_nembed;
our $verbose;
our $o_nfonts;
our $o_afonts;
our $o_nopp;
our $o_oencoding;

our $g_file;
our $g_file_name;

my $file_tmp;		# temporary page (w/o resources)
my $file_sps;		# reduced fonts to be embedded (u2ps_rs)
my $file_out;		# final output file

sub init_files # (infile, outfile = undef)
{
	my $file = shift || die "File name required\n";
	my $out = shift;

	$g_file = $file;
	if(defined($out)) {
		$file_tmp = "$out.tmp.ps";
		$file_sps = "$out.tmp.sps";
		$file_out = $out;
	} else {
		$file_tmp = "$file.tmp.ps";
		$file_sps = "$file.tmp.sps";
		$file_out = "$file.ps";
	}
	$g_file_name = decode($o_oencoding, $g_file, Encode::FB_DEFAULT);

	open(FILE, "<", $file) || die "Can't open $file: $!\n";
	open(PAGE, ">", $file_tmp) || die "Can't open $file_tmp: $!\n";
	binmode(PAGE, ":utf8");
	select(PAGE);
}

sub fini_files
{
	close(PAGE);
	close(FILE);
}

sub run_pp_rs
{
	my @pp_opts = ();
	push(@pp_opts, "-o$file_out");
	my @rs_opts = ();
	push(@rs_opts, "-G$o_gs") if defined $o_gs;
	push(@rs_opts, "-K") if $o_keep;

	if($o_nembed || $o_nopp) {
		warn "Skipping resources\n" if $verbose;
		rename($file_tmp, $file_out) || die "Can't rename $file_tmp to $file_out: $!\n";
	} else {
		push(@pp_opts, "-Nfont", "-Dfonts") if $o_nfonts;
		push(@pp_opts, "-Sfonts=$file_sps");
		push(@pp_opts, '-v') if $verbose;
		push(@rs_opts, "-F") if $o_nfonts;
		push(@rs_opts, "-A") if $o_afonts;

		warn "Running u2ps_rs\n" if $verbose;
		runcheck("$b_px/u2ps_rs.px", @rs_opts, $file_tmp);
		warn "Running u2ps_pp\n" if $verbose;
		runcheck("$b_px/u2ps_pp.px", @pp_opts, $file_tmp);
		warn "Removing temporary files\n" if $verbose && !$o_keep;
		unlink($file_sps) unless $o_keep;
		unlink($file_tmp) unless $o_keep;
	}

}

sub runcheck
{
	warn(join(" ", @_)."\n") if $verbose;

	my $ret = system(@_);
	return if $ret == 0;
	my $cmd = shift;
	my $reason;

	if(WIFEXITED($ret)) {
		$reason = sprintf('exit status %i', WEXITSTATUS($ret));
	} elsif(WIFSIGNALED($ret)) {
		$reason = sprintf('killed by signal %i', WTERMSIG($ret));
	} elsif(WIFSTOPPED($ret)) {
		$reason = 'stopped';
	} else {
		$reason = 'reason unknown';
	}

	if($verbose) {
		warn 'failed: '.join(' ', $cmd, @_)."\n";
		warn "failed: $reason\n";
		exit(-1);
	} else {
		die "$cmd failed, $reason\n";
	}
}

1;
