# page headings handling

use strict;
use utf8;

our $o_headings;
our $o_title;
our %o_hd;
our $o_bookish;

our $g_file_name;
our $c_page;

my @headings;

sub prepare_headings
{
	my $setup = shift;
	my $fsize = shift;

	if($o_headings || defined($o_title)) {
		$o_hd{'hl'} = headline_escape($o_title)
				if !defined($o_hd{'hl'}) && defined $o_title;
		$o_hd{'hl'} = $g_file_name unless defined $o_hd{'hl'};
		$o_hd{'hr'} = '%p' unless defined $o_hd{'hr'};
	};
	$o_title = $g_file_name unless defined $o_title;

	foreach my $k (keys %o_hd) {
		delete $o_hd{$k} unless defined $o_hd{$k};
	}
}

sub head_setup
{
	my $setup = shift;

	@headings = ( );
	push @$setup, "\n";
	push @$setup, "/page-headings {\n";
	push @$setup, "\tpage-headings-init bookish and\n";
	foreach my $k (keys %o_hd) {
		my $hk = $o_hd{$k};
		if(headline_has_variables($hk)) {
			# variable header
			push(@$setup, sprintf("\texch 1 index head-%s\n", $k));
			unshift(@headings, $k);
		} else {
			# constant header
			$hk = headline_subst($hk);
			push(@$setup, sprintf("\t%s 1 index head-%s\n", psstring($hk), $k));
		}
	}
	push @$setup, "\tpop page-headings-fini\n";
	push @$setup, "} def\n";
}

sub page_headings
{
	foreach my $h (@headings) {
		print_c(psstring(headline_subst($o_hd{$h}))." ");
	}
	print_c($c_page % 2 ? 'false ' : 'true ');
}

# ------------------------------------------------------------------------------

sub headline_has_variables
{
	my $str = shift;
	return ($str =~ /%p/);
}

sub headline_subst
{
	my %chars = ('T' => $o_title, 'p' => $c_page, '%' => '%');
	my $str = shift;
	$str =~ s/%(.)/ifdef($chars{$1}, '')/ge;
	return $str;
}

sub headline_escape
{
	my $str = shift;
	$str =~ s/%/%%/g;
	return $str;
}

1;
