# Utility routines working within a single page; called from
# either _rawt.pl of _ansi.pl

use Text::CharWidth qw(mbwidth);
use utf8;

use strict;

our %o_hd;
our $o_wrap;
our $o_lstart;
our $verbose;
our $o_lnum;
our $o_ts;
our $o_no_ansi;

our $g_cols;
our $g_lines;

our $c_page;
our $c_soft_line;
our $c_hard_line;
our $c_soft_col;

my $c_post_llen = 0;
my $ps_llimit = 100;

# page_headings(void)

sub new_page
{
	my $lref = $c_hard_line + $o_lstart;

	$c_page++;
	$c_soft_col = 0;
	$c_soft_line = 0;

	print "\nshowpage\n" if $c_page > 1;
	print "\n";
	print "%%Page: $c_page $c_page\n";
	print_c("$lref page-line\n") if $o_lnum;
	page_headings() if keys %o_hd;
	print_c("page-init\n");
	page_setattrs() unless $o_no_ansi;
	print "\n";
}

# Print a chunk of text, possibly starting new line.
# All wrapping occurs here
sub put_text
{
	my $str = shift;
	#warn "wrapping if off\n" unless $o_wrap and $verbose;
	return print_t($str) unless $o_wrap;
	
	warn "put_text with soft-line=$c_soft_line\n" if $c_soft_line > $g_lines;

	#warn "trying to wrap line (width=$g_cols, col=$c_soft_col, softline=$c_soft_line)\n" if $verbose;
	my @str = split(//, $str);
	my $tmp = '';

	foreach my $c (@str) {
		my $w = mbwidth($c);
		if($c_soft_col > 0 && $c_soft_col + $w > $g_cols) {
			print_t($tmp) if length($tmp) > 0;
			print_c("n\n");
			print_c("  ") if $o_lnum;
			$c_soft_line++;
			new_page() if $c_soft_line % $g_lines == 0;
			$tmp = $c;
			$c_soft_col = $w;
			warn "\tgot a n-linebreak, soft line = $c_soft_line\n" if $verbose;
		} else {
			$tmp .= $c;
			$c_soft_col += $w;
		}
	}
	#warn "\tdone, col=$c_soft_col\n" if $verbose;
	print_t($tmp) if length($tmp) > 0;
}

sub new_line
{
	my $pagebreak = shift;

	new_page() if $pagebreak || ($c_soft_line % $g_lines == 0);
	$c_soft_col = 0;
	print_c($pagebreak ? "l " : "L ") if $o_lnum;
}

sub end_line
{
	$c_hard_line++;
	$c_soft_line++;
	print_c("N\n");
}

sub put_tab
{
	print_c("T ");
	my $mod = $c_soft_col % $o_ts;
	$c_soft_col += ($mod == 0) ? $o_ts : ($o_ts - $mod);
}

sub put_cr
{
	print_c("CR\n");
	$c_soft_col = 0;
}

sub put_bs
{
	return if $c_soft_col == 0;
	print_c("BS ");
	$c_soft_col--;
}

sub usplitwidth
{
	my $max = shift;
	my $str = shift;
	my $len = shift || 0;
	my @str = split(//, $str);
	my @ret = '';
	my $ret;

	foreach my $c (@str) {
		my $w = mbwidth($c);
		if($len > 0 && $len + $w > $max) {
			push(@ret, [ $ret, $len ]);
			$ret = $c;
			$len = $w;
		} else {
			$ret .= $c;
			$len += $w;
		}
	}
	push(@ret, [ $ret, $len ]);

	return @ret;
}

sub print_t
{
	my $text = shift;
	$text =~ s/([()\\])/\\$1/g;
	print_c(sprintf("(%s)u ", $text));
}

# print raw piece of postscrip code, wrapping *postscript* line
# if needed
sub print_c
{
	my $chunk = shift;
	my $len;

	if($chunk =~ /([^\n]*)\n/) {
		$len = length($1);
	} else {
		$len = length($chunk);
	}

	if($c_post_llen > 0 && $c_post_llen + $len > $ps_llimit) {
		print "\n";
		#print "% chunk too long: llen=$llen len=$len limit=$ps_llimit\n";
		$c_post_llen = 0
	}	

	print $chunk; 

	if($chunk =~ /\n([^\n]*)$/) {
		$c_post_llen = length($1);
	} else {
		$c_post_llen += length($chunk);
	}
}

1;
